@@ -97,6 +97,11 @@ QR::init (const Matrix& a, QR::type qr_type)
 	      for (int i = limit + 1; i < m; i++)
 		A_fact.elem (i, j) *= tau.elem (j);
 	    }
+
+	  r = A_fact;
+
+	  if (m > n)
+	    r.resize (m, n);
 	}
       else
 	{
