@@ -40,7 +40,7 @@ function oneplot ()
       gset size 1, 1;
       gset origin 0, 0;
       __multiplot_mode__ = 0;
-      gnuplot_command_replot "rep";
+      gnuplot_command_replot = "rep";
       clearplot;
     endif
 
