@@ -27,8 +27,17 @@
 
 ## Author: jwe
 
-function ylabel (...)
+function ylabel (text)
 
-  __axis_label__ ("ylabel", all_va_args);
+  if (nargin != 1)
+    usage ("ylabel (text)");
+  endif
+
+  if (isstr (text))
+    command = sprintf ("gset ylabel \"%s\"", text);
+    eval (command);
+  else
+    error ("ylabel: text must be a string");
+  endif
 
 endfunction
