@@ -70,7 +70,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   static builtin_function S ## name (#name, is_text_fcn, F ## name, doc)
 
 #define DEFINE_FUN_STRUCT_FUN(name, is_text_fcn, doc) \
-  builtin_function& \
+  builtin_function * \
   FS ## name (void) \
   { \
     static builtin_function *retval = 0; \
@@ -78,7 +78,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     if (! retval) \
       retval = new builtin_function (#name, is_text_fcn, F ## name, doc); \
  \
-    return *retval; \
+    return retval; \
   }
 
 #define DECLARE_FUN(name, args_name, nargout_name) \
