@@ -78,6 +78,7 @@ Free Software Foundation, Inc.
 #include "sighandlers.h"
 #include "symtab.h"
 #include "sysdep.h"
+#include "systime.h"
 #include "unwind-prot.h"
 #include "utils.h"
 #include "variables.h"
@@ -92,7 +93,7 @@ static string Vps2;
 string Vps4;
 
 // The time we last printed a prompt.
-time_t Vlast_prompt_time;
+double Vlast_prompt_time;
 
 // Character to append after successful command-line completion attempts.
 static char Vcompletion_append_char;
@@ -498,7 +499,17 @@ octave_gets (void)
 {
   char *retval = 0;
 
-  Vlast_prompt_time = time (0);
+#if defined (HAVE_GETTIMEOFDAY)
+  struct timeval tp;
+#if defined  (GETTIMEOFDAY_NO_TZ)
+  gettimeofday (&tp);
+#else
+  gettimeofday (&tp, 0);
+#endif
+  Vlast_prompt_time = (double) tp.tv_sec + tp.tv_usec / 1e6;
+#else
+  Vlast_prompt_time = (double) time (0);
+#endif
 
   if ((interactive || forced_interactive)
       && (! (reading_fcn_file || reading_script_file)))
@@ -529,12 +540,18 @@ octave_gets (void)
       if (! input_from_startup_file)
 	octave_command_history.add (current_input_line);
 
-      octave_diary << current_input_line;
+      if (! (reading_fcn_file || reading_script_file))
+	{
+	  octave_diary << current_input_line;
+
+	  if (current_input_line[current_input_line.length () - 1] != '\n')
+	    octave_diary << "\n";
+	}
 
       do_input_echo (current_input_line);
     }
-
-  octave_diary << "\n";
+  else if (! (reading_fcn_file || reading_script_file))
+    octave_diary << "\n";
   
   return retval;
 }
