@@ -26,7 +26,6 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define octave_input_h 1
 
 #include <cstdio>
-#include <ctime>
 
 #include <string>
 
@@ -75,7 +74,7 @@ char *gnu_readline (const char *s, bool force_readline = false);
 
 extern string Vps4;
 
-extern time_t Vlast_prompt_time;
+extern double Vlast_prompt_time;
 
 extern void symbols_of_input (void);
 
