@@ -203,7 +203,7 @@ kluge_procbuf_delay (void)
   return -1;
 }
 
-static void
+void
 symbols_of_oct_procbuf (void)
 {
   DEFVAR (kluge_procbuf_delay, Vkluge_procbuf_delay, 0, kluge_procbuf_delay,
