@@ -71,7 +71,7 @@ public:
 
   void stash_fcn_file_name (void);
 
-  void mark_fcn_file_up_to_date (time_t t)
+  void mark_fcn_file_up_to_date (double t)
     { t_checked = t; }
 
   void stash_fcn_file_time (time_t t)
@@ -86,7 +86,7 @@ public:
   time_t time_parsed (void)
     { return t_parsed; }
 
-  time_t time_checked (void)
+  double time_checked (void)
     { return t_checked; }
 
   void mark_as_system_fcn_file (void);
@@ -155,7 +155,7 @@ private:
 
   // The time the file was last checked to see if it needs to be
   // parsed again.
-  time_t t_checked;
+  double t_checked;
 
   // True if this function came from a file that is considered to be a
   // system function.  This affects whether we check the time stamp
