@@ -312,45 +312,6 @@ has occured, and MSG contains a system-dependent error message.")
   return retval;
 }
 
-DEFUN (vfork, args, ,
- "[PID, MSG] = vfork ()\n\
-\n\
-Create a copy of the current process.\n\
-\n\
-If successful, PID is either the process ID and you are in the parent,\n\
-or 0, and you are in the child.  If PID is less than zero, an error\n\
-has occured, and MSG contains a system-dependent error message.")
-{
-  octave_value_list retval;
-
-  retval(1) = string ();
-  retval(0) = -1.0;
-
-  int nargin = args.length ();
-
-  if (nargin == 0)
-    {
-#if defined (HAVE_VFORK) || defined (HAVE_FORK)
-#if defined (HAVE_VFORK)
-      pid_t pid = vfork ();
-#else
-      pid_t pid = fork ();
-#endif
-
-      retval(0) = (double) pid;
-
-      if (pid < 0)
-	retval(1) = strerror (errno);
-#else
-      gripe_not_supported ("vfork");
-#endif
-    }
-  else
-    print_usage ("vfork");
-
-  return retval;
-}
-
 DEFUN (getpgrp, args, ,
   "pgid = getpgrp (): return the process group id of the current process")
 {
