@@ -339,7 +339,7 @@ script file but without requiring the file to be named `FILE.m'.")
 	{
 	  file = oct_tilde_expand (file);
 
-	  parse_fcn_file (1, file);
+	  parse_fcn_file (file, true, true);
 
 	  if (error_state)
 	    error ("source: error sourcing file `%s'", file.c_str ());
@@ -356,7 +356,7 @@ script file but without requiring the file to be named `FILE.m'.")
 // Fix up things before exiting.
 
 void
-clean_up_and_exit (int retval)
+clean_up_for_exit (void)
 {
 #if defined (USE_READLINE)
   rl_deprep_terminal ();
@@ -372,18 +372,16 @@ clean_up_and_exit (int retval)
 
   cleanup_tmp_files ();
 
-  if (!quitting_gracefully && (interactive || forced_interactive))
+  if (! quitting_gracefully && (interactive || forced_interactive))
     cout << "\n";
+}
 
-  if (retval == EOF)
-    retval = 0;
+void
+clean_up_and_exit (int retval)
+{
+  clean_up_for_exit ();
 
-  exit (retval);
-
-  // This is bogus but should prevent g++ from giving a warning saying
-  // that this volatile function does return.
-
-  panic_impossible ();
+  exit (retval == EOF ? 0 : retval);
 }
 
 DEFUN_TEXT (casesen, args, ,
