@@ -26,7 +26,6 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <cfloat>
 #include <cmath>
-#include <ctime>
 #include <cstdio>
 #include <cstring>
 
@@ -81,6 +80,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "pt-plot.h"
 #include "pr-output.h"
 #include "syscalls.h"
+#include "systime.h"
 #include "toplev.h"
 #include "unwind-prot.h"
 #include "utils.h"
@@ -425,7 +425,7 @@ symbol_out_of_date (symbol_record *sr)
 	      && ! (Vignore_function_time_stamp
 		    && ans->is_system_fcn_file ()))
 	    {
-	      if (ans->time_checked () < Vlast_prompt_time)
+	      if (Vlast_prompt_time >= ans->time_checked ())
 		{
 		  time_t tp = ans->time_parsed ();
 
@@ -436,7 +436,18 @@ symbol_out_of_date (symbol_record *sr)
 		  else
 		    fname = fcn_file_in_path (ff);
 
-		  ans->mark_fcn_file_up_to_date (time (0));
+#if defined (HAVE_GETTIMEOFDAY)
+		  struct timeval tv;
+#if defined  (GETTIMEOFDAY_NO_TZ)
+		  gettimeofday (&tv);
+#else
+		  gettimeofday (&tv, 0);
+#endif
+		  double now = (double) tv.tv_sec + (double) tv.tv_usec / 1e6;
+#else
+		  double now = (double) time (0);
+#endif
+		  ans->mark_fcn_file_up_to_date (now);
 
 		  file_stat fs (fname);
 
